
#include "FastLED.h"

#define NUM_LEDS 101
#define DATA_PIN 3
CRGB leds[NUM_LEDS];

#define audio_latency 200 // Compensate for latency of software synth.  Usually this can be found in the settings for buffer size.
// In addition to buffer size, there may be audio processing time, and it depends on whether the software synth sends USB MIDI before or after it processes and buffers.
//  To be more accurate, you can use synth latency + feet from speaker to ears - USB MIDI transfer time.  I think USB MIDI transfer time is about 1mS on average.  This might vary with operating system and software. 
#define max_buffered_notes 10 // How many of the same notes can occur during latency of software synth?  Higher numbers increase RAM usage.
#define indicator_LED_pin 16 // To turn off indicator LED, select an unused pin.
#define lowest_note 16  // First light leds[0] uses what MIDI number?  16 (E) is the first above 20Hz.  88-key piano starts at 21 (A).  This number is subtracted from MIDI note number to calculate leds[] index number.
// If USB MIDI sends a note below lowest_note or above lowest_note + NUM_LEDS - 1, that note will be ignored.  To share lights with multiple notes, see DMX sketches.

#define velocity_sensitive_brightness 1 // brightness varies with velocity?
#define volumeBS 2 // velocity is bitshifted left this many times, + 1 to calculate brightness, capped at 255 max.
#define relative_curve 0 // With velocity sensitive brightness, is the entire curve used proportionally, or is only the lower part used?
#define velocity_sensitive_sustain 0 // sustain varies with velocity?  Usually yes for acoustic sound, no for synth. 
#define fadeup 1 // fadeup? or 0 is instant on
#define fadeup_BS 5 // 2^4 == 16 mS
#define fadeup_time 32 //  must agree with fadeup_BS
#define note_on_sustain 1 // hold brightness?  Usually yes for synth, no for acoustic sound.
#define noteon_sustain_at  0 // hold brightness after noteon_sustain_at milliseconds. (after attack)
#define fadeoff_noteoff 1 // fadeoff or turnoff when key is lifted?
#define noteon_fade_bitshift 4 // 0: max 255mS on full velocity, 1: max 510mS, 2: max 1020mS, 3: max 2040mS, 4: max 4080mS, 5: max 8160mS
#define noteoff_fade_bitshift 1 // 0: max 255mS on full velocity, 1: max 510mS, 2: max 1020mS, 3: max 2040mS, 4: max 4080mS, 5: max 8160mS

#define ignore_percussion 0 //  Yes: 1, No: 0
#define percussion_channel 10 //  Usually it's 10.  Look at MIDI file. 

byte noteon[NUM_LEDS]; // noteon: 1 or noteoff: 0  Can use all bits to save RAM.  Requires bitwise math.  Minimal saving of RAM in this sketch, probably not worth the complication.
byte current_brightness_index[NUM_LEDS];
byte start_brightness_index[NUM_LEDS];
long current_millis;
long start_millis[NUM_LEDS];

long scheduled_noteon_start_millis[NUM_LEDS][max_buffered_notes];
long scheduled_noteoff_start_millis[NUM_LEDS][max_buffered_notes];
byte scheduled_start_brightness_index[NUM_LEDS][max_buffered_notes];

unsigned long indicator_LED_start_millis;
unsigned long indicator_LED_end_millis;

const PROGMEM uint16_t hue[128] =
{0,0,0,0,0,0,0,0,0,0,0,0,
0, 256, 512, 1024, 1536, 2048, 2389, 2731, 3072, 3413, 3755, 3840,  // If green dominates too much over red: smaller steps near red, longer steps near green.
0, 256, 512, 1024, 1536, 2048, 2389, 2731, 3072, 3413, 3755, 3840,  // Avoid using modulo function by using larger array of repetitive data.
0, 256, 512, 1024, 1536, 2048, 2389, 2731, 3072, 3413, 3755, 3840,
0, 256, 512, 1024, 1536, 2048, 2389, 2731, 3072, 3413, 3755, 3840,
0, 256, 512, 1024, 1536, 2048, 2389, 2731, 3072, 3413, 3755, 3840,
0, 256, 512, 1024, 1536, 2048, 2389, 2731, 3072, 3413, 3755, 3840,
0, 256, 512, 1024, 1536, 2048, 2389, 2731, 3072, 3413, 3755, 3840,
0, 256, 512, 1024, 1536, 2048, 2389, 2731, 3072, 3413, 3755, 3840,
0, 256, 512, 1024, 1536, 2048, 2389, 2731, 3072, 3413, 3755, 3840,
0, 256, 512, 1024, 1536, 2048, 2389, 2731};  

const PROGMEM uint8_t curve[256] = {  // nonlinear dimming and mixing curve
0	,
1	,
1	,
1	,
1	,
1	,
1	,
2 ,
2	,
2	,
2	,
2	,
3	,
3	,
3	,
3	,
3	,
4	,
4	,
4	,
4	,
5 ,
5	,
5	,
5	,
6	,
6	,
6	,
6	,
7 ,
7	,
7	,
7	,
8 ,
8 ,
8	,
9	,
9	,
9	,
10	,
10	,
10	,
11	,
11	,
11	,
12	,
12	,
12	,
13	,
13	,
14	,
14	,
15	,
15	,
16	,
16	,
17	,
17	,
18	,
18	,
19	,
19	,
20	,
20	,
21	,
21	,
22	,
23	,
23	,
24	,
24	,
25	,
25	,
26	,
27	,
27	,
28	,
29	,
29	,
30	,
31	,
31	,
32	,
33	,
33	,
34	,
35	,
35	,
36	,
37	,
38	,
38	,
39	,
40	,
41	,
41	,
42	,
43	,
44	,
44	,
45	,
46	,
47	,
48	,
49	,
49	,
50	,
51	,
52	,
53	,
54	,
55	,
55	,
56	,
57	,
58	,
59	,
60	,
61	,
62	,
63	,
64	,
65	,
66	,
67	,
68	,
69	,
70	,
71	,
72	,
73	,
74	,
75	,
76	,
77	,
78	,
79	,
80	,
81	,
82	,
83	,
84	,
85	,
86	,
88	,
89	,
90	,
91	,
92	,
93	,
94	,
96	,
97	,
98	,
99	,
100	,
101	,
103	,
104	,
105	,
106	,
108	,
109	,
110	,
111	,
113	,
114	,
115	,
116	,
118	,
119	,
120	,
122	,
123	,
124	,
126	,
127	,
128	,
130	,
131	,
132	,
134	,
135	,
136	,
138	,
139	,
141	,
142	,
144	,
145	,
146	,
148	,
149	,
151	,
152	,
154	,
155	,
157	,
158	,
160	,
161	,
163	,
164	,
166	,
167	,
169	,
170	,
172	,
173	,
175	,
177	,
178	,
180	,
181	,
183	,
185	,
186	,
188	,
190	,
191	,
193	,
194	,
196	,
198	,
199	,
201	,
203	,
205	,
206	,
208	,
210	,
211	,
213	,
215	,
217	,
218	,
220	,
222	,
224	,
226	,
227	,
229	,
231	,
233	,
235	,
236	,
238	,
240	,
242	,
244	,
246	,
247	,
249	,
251	,
253	,
255};


